/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.SessionState;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.IdpSample.cs.slo {

	/// <summary>
	/// SLOIdpHandler is invoked for logout on IdP side. This is initiated by
	/// the user from IdP or SP Sample Applications.
	/// Pingfederate server calls SLOIdpHandler if the user starts SLO from SP
	/// side, or it is called by the IdP Sample Application if SLO starts from 
	/// IdP side.
	/// </summary>
	public class SLOIdpHandler : IHttpHandler, IRequiresSessionState {

		private Links samlLinks;

		public void ProcessRequest(HttpContext context) {

			HttpRequest  request     = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(session[Constants.IDP_CONFIG_TABLE] == null) {
				idpConfigTable = GetIdpConfigProps(context);
				session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else {
				idpConfigTable = (Hashtable)session[Constants.IDP_CONFIG_TABLE];
			}
			context.Trace.Warn("idpConfigTable has been loaded");

			samlLinks = new Links(idpConfigTable, request);

			// Create url for redirect after performing logout
			String resumePath = request[Constants.RESUME_PATH];
			String returnURL = createRedirectURL(request);
			context.Trace.Warn("redirectURL = " + returnURL);

			//Get user attributes from session
			Hashtable userInfo = (Hashtable)session[Constants.USER_INFO];

			if((userInfo == null) || (userInfo.Count <= 0)) {
				context.Trace.Warn("userInfo = null, redirect to login");
				String loginPage = samlLinks.getIdpLoginPageUrl(Constants.LOCAL_TYPE);
				loginPage += "&error=Local session does not exist.";
				response.Redirect(loginPage);
				return;
			}

			// Send Logout request via PFToken cookies in case we
			// have got the response from PF server
			if(request.QueryString.Get(Constants.RESUME_PATH) != null) {
				// Send Logout request via PFToken cookie or query
				returnURL = setLogoutPFToken(context, returnURL, userInfo, true);
				session.Abandon();
			}
			// Redirect to the url created earlier
			response.Redirect(returnURL);
		}

		/// <summary>
	   /// createRedirectURL method checks to see if the request contains resume path.
	   /// This indicates that the request came from PingFederate server and the 
	   /// resume path will be used for the response url. If there is no resume path,
	   /// the response url is obtained from the servlet configuration and it is the
	   /// url to the SLO service for Idp Sample Application
	   /// </summary>
	   /// <param name="request">current user request</param>
	   /// <returns>url to redirect</returns>
		private String createRedirectURL(HttpRequest request) {

			// Get resume path from request
			String urlToSpApp = request.QueryString.Get(Constants.RESUME_PATH);
		
			if(urlToSpApp == null) {
				// There is no resume path so use the url to Idp SLO service
						
				// This is the URL for IDP-initiated SLO			
				urlToSpApp = samlLinks.getIdpSLOLink();
			}
			else {
				urlToSpApp = samlLinks.getResumePathUrl(urlToSpApp);  //create full path
			}
			return urlToSpApp;
		}

	
		/// <summary>
		/// setLogoutPFToken method uses the Java Integration Kit API to create an 
		/// instance of Idp Agent. The user attributes are added to this agent and 
		/// then the logout PFTOKEN is created
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <param name="context">url for redirect after performing logout</param>
		/// <param name="userInfo">Hashtable of user attributes</param>
		/// <param name="logoutSuccessful">flag indicating successful logout</param>
		/// <returns>URL with pftoken logout parameter</returns>
		private String setLogoutPFToken(HttpContext context, String url, Hashtable userInfo,
				bool logoutSuccessful) {

			HttpRequest  request  = context.Request;
			HttpResponse response = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(session[Constants.IDP_CONFIG_TABLE] == null) 
			{
				idpConfigTable = GetIdpConfigProps(context);
				session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else 
			{
				idpConfigTable = (Hashtable)session[Constants.IDP_CONFIG_TABLE];
			}
			context.Trace.Warn("idpConfigTable has been loaded");
	
			//read properties from Sp pfagent-properties file
			String idpProps  = ConfigurationSettings.AppSettings["pfagent-properties"];
			String propsPath = context.Server.MapPath("config/" + idpProps);

			//get password
			String password = (String)idpConfigTable[Constants.PASSWORD];

			//get holder name
			String holderName = (String)idpConfigTable[Constants.HOLDER_NAME];

			// Create PFToken Idp Agent
			PftokenAgent idpAgent = new PftokenAgent(password, holderName);

			// Add user attributes to the agent
			idpAgent.AddAttributes(userInfo);

			// Create logout PFToken
			return idpAgent.ConstructLogoutPFToken(request, response, url, logoutSuccessful);
		}
	 
		/// <summary>
		/// Reads general configuration properties XML file for the
		/// IDP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetIdpConfigProps(HttpContext context) {

			//read properties from Idp config XML file
			String idpConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + idpConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable idpConfigTable = xmlUtility.GetConfigProps(configPath);
			if(idpConfigTable == null) {
				String errorMessage = "Error in " + idpConfig;
				context.Response.Redirect(Constants.ERROR_PAGE.Substring(1) + errorMessage);
			}
			return idpConfigTable;
		}

		public bool IsReusable {
			get { return true; }
		}
	}
}