/*******************************************************************************
 * Copyright (C) 2006 Ping Identity Corporation All rights reserved.
 * 
 * This software is licensed under the Open Software License v2.1 (OSL v2.1).
 * 
 * A copy of this license has been provided with the distribution of this
 * software. Additionally, a copy of this license is available at:
 * http://opensource.org/licenses/osl-2.1.php
 *  
 ******************************************************************************/
using System;
using System.Collections;
using System.Configuration;
using System.IO;
using System.Web;
using System.Web.SessionState;
using Com.PingIdentity.Adapters.Pftoken.Core;
using Com.PingIdentity.Adapters.Pftoken.AuthnContext;
using Com.PingIdentity.PingFederate.SampleApp.SampleAppUtility;

namespace Com.PingIdentity.PingFederate.SampleApp.IdpSample.cs.sso {

	/// <summary>
	/// SSOIdpHandler is used for creating user attributes and sending them
	/// via PFTOKEN cookie or query parameter to the SP Sample Application.
	/// Pingfederate server calls SSOIdpHandler if the user starts SSO from SP
	/// side, or it is called directly by the user if SSO starts from IdP side.
	/// </summary>
	public class SSOIdpHandler : IHttpHandler, IRequiresSessionState {

		private Links samlLinks;

		public void ProcessRequest(HttpContext context) {

			HttpRequest  request     = context.Request;
			HttpResponse response    = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(session[Constants.IDP_CONFIG_TABLE] == null) {
				idpConfigTable = GetIdpConfigProps(context);
				session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else {
				idpConfigTable = (Hashtable)session[Constants.IDP_CONFIG_TABLE];
			}
			context.Trace.Warn("idpConfigTable has been loaded");

			//get properties from Idp connections file
			ArrayList idpConnectionsList;
			if(session[Constants.IDP_CONNECT_LIST] == null) {
				idpConnectionsList = GetIdpConnectionsProps(context);
				session[Constants.IDP_CONNECT_LIST] = idpConnectionsList;
			}
			else {
				idpConnectionsList = (ArrayList)session[Constants.IDP_CONNECT_LIST];
			}
			context.Trace.Warn("idpConnectionsList has been loaded");

			samlLinks = new Links(idpConfigTable, request);

			String urlToSpApp = createRedirectURL(request);
			context.Trace.Warn("redirectURL = " + urlToSpApp);

			//create URL to Login Page
			String loginPage = samlLinks.getIdpLoginPageUrl(Constants.GLOBAL_TYPE);

			//create URL to Error Page
			String errorPage = samlLinks.getIdpErrorPageUrl();

			//Get user attributes from session
			Hashtable userInfo = (Hashtable)session[Constants.USER_INFO];
			bool infoExists = false;
			if((userInfo != null) && (userInfo.Count > 0)) {
				infoExists = true;
			}

			//get ForceAuthn and IsPassive parameters values
			bool forceFlag   = getForceAuthnValue(request);
			bool passiveFlag = getIsPassiveValue(request);

			// Check to see if login is required
			if( (forceFlag && !passiveFlag) || (!infoExists && !passiveFlag) ) {
				// In this case we have to force the user to do login
				String resumePath = request.QueryString.Get(Constants.RESUME_PATH);
				loginPage = samlLinks.addParameter(loginPage, Constants.RESUME_PATH, resumePath);
				session.Abandon();
				context.Trace.Warn("redirect to loginPage");
				response.Redirect(loginPage);
				return;
			}

			//send the user to the error.aspx
			if( (forceFlag && passiveFlag) || (!infoExists && passiveFlag) ) {
				// This will be an error if both flags are set or the IsPassive flag is true 
				// when there are no user attributes
				context.Trace.Warn("redirect to login page with message");
				String errorMessage = "&error=SSO Error, IsPassive is true";
				response.Redirect(loginPage + errorMessage);
				return;
			}

			// The user has been authenticated so create PFToken
			urlToSpApp = setPFToken(context, urlToSpApp, userInfo);

			// Redirect to url obtained earlier
			response.Redirect(urlToSpApp);
		}
		
		/// <summary>
		/// createRedirectURL method checks to see if the request contains resume path. This
		/// indicates that the request came from PingFederate server and the resume 
		/// path will be used for the response url. If there is no resume path, the 
		/// response url is obtained from the servlet configuration and it is the url
		/// to the SSO service for Idp Sample Application
		/// </summary>
		/// <param name="request">current user request</param>
		/// <returns>url to redirect</returns>
		private String createRedirectURL(HttpRequest request) {

			// Get resume path from request
			String urlToSpApp = request.QueryString.Get(Constants.RESUME_PATH);
		
			if(urlToSpApp == null) {
				// Get the SP Partner Id and Target Resource from request
				String partnerSpId    = request.QueryString.Get(Constants.PARTNER_SP_ID);
				String targetResource = request.QueryString.Get(Constants.TARGET_RESOURCE);
						
				// This is the URL for IDP-initiated SSO			
				urlToSpApp = samlLinks.getIdpSSOLink(partnerSpId, targetResource);
			}
			else {
				urlToSpApp = samlLinks.getResumePathUrl(urlToSpApp);  //create full path
			}
			return urlToSpApp;
		}

		/// <summary>
		/// setPFToken method uses the Java Integration Kit API to create an 
		/// instance of Idp Agent. The user attributes are added to this agent and 
		/// then the PFToken is created as a cookie or query parameter
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <param name="url">url to append pftoken</param>
		/// <param name="userInfo">Hashtable of user attributes</param>
		/// <returns>URL with pftoken</returns>
		private String setPFToken(HttpContext context, String url, Hashtable userInfo) 
		{
			String returnUrl = url;

			HttpRequest  request  = context.Request;
			HttpResponse response = context.Response;
			HttpSessionState session = context.Session;

			//get properties from Idp config file
			Hashtable idpConfigTable;
			if(session[Constants.IDP_CONFIG_TABLE] == null) 
			{
				idpConfigTable = GetIdpConfigProps(context);
				session[Constants.IDP_CONFIG_TABLE] = idpConfigTable;
			}
			else 
			{
				idpConfigTable = (Hashtable)session[Constants.IDP_CONFIG_TABLE];
			}
			context.Trace.Warn("idpConfigTable has been loaded");
	
			//read properties from Idp pfagent-properties file
			String idpProps  = ConfigurationSettings.AppSettings["pfagent-properties"];
			String propsPath = context.Server.MapPath("config/" + idpProps);

			//get password
			String password = (String)idpConfigTable[Constants.PASSWORD];

			//get holder name
			String holderName = (String)idpConfigTable[Constants.HOLDER_NAME];

			// Create PFToken Idp Agent
			PftokenAgent idpAgent = new PftokenAgent(password, holderName);
		
			// Add user attributes to the agent		
			idpAgent.AddAttributes(userInfo);

			// Create PFToken
			returnUrl = idpAgent.ConstructPFToken(request, response, url);
			return returnUrl;
		}

		/// <summary>
		/// getForceAuthnValue method retrieves the ForceAuthn parameter from
		/// request and returns the flag based on this value.
		/// </summary>
		/// <param name="request">current user request</param>
		/// <returns>bool forceFlag</returns>
		private bool getForceAuthnValue(HttpRequest request) {

			// Retrieve parameter from request
			String forceAuthn = request.QueryString.Get(Constants.FORCE_AUTH_PARAM);

			bool forceFlag = false;
			// Convert string parameters to boolean
			if( forceAuthn != null &&
					String.Compare(forceAuthn, Constants.TRUE_STRING_VALUE, true) == 0 ) {
				forceFlag = true;
			}
			return forceFlag;
		}

		/// <summary>
		/// getIsPassiveValue method retrieves the IsPassive parameter from
		/// request and returns the flag based on this value.
		/// </summary>
		/// <param name="request">current user request</param>
		/// <returns>bool passiveFlag</returns>
		private bool getIsPassiveValue(HttpRequest request) {

			// Retrieve parameter from request
			String isPassive = request.QueryString.Get(Constants.IS_PASSIVE_PARAM);

			bool passiveFlag = false;
			// Convert string parameters to boolean
			if( isPassive != null &&
				String.Compare(isPassive, Constants.TRUE_STRING_VALUE, true) == 0 ) {
				passiveFlag = true;
			}
			return passiveFlag;
		}

		/// <summary>
		/// Reads general configuration properties XML file for the
		/// IDP Sample Application and converts it to Hastable object
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>Hashtable of user general configuration properties</returns>
		private Hashtable GetIdpConfigProps(HttpContext context) {

			//read properties from Idp config XML file
			String idpConfig  = ConfigurationSettings.AppSettings["config-file"];
			String configPath = context.Server.MapPath("config/" + idpConfig);

			XmlUtility xmlUtility = new XmlUtility();
			Hashtable idpConfigTable = xmlUtility.GetConfigProps(configPath);
			if(idpConfigTable == null) {
				String errorMessage = "Error in " + idpConfig;
				context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
			}
			return idpConfigTable;
		}

		/// <summary>
		/// Reads connection properties XML file for specifying Sp Connections
		/// and converts it to ArrayList of Hashtables with user SP connections
		/// </summary>
		/// <param name="context">current HttpContext</param>
		/// <returns>ArrayList of user connection properties</returns>
		private ArrayList GetIdpConnectionsProps(HttpContext context) {

			//read properties from Idp connections XML file
			String idpConnections  = ConfigurationSettings.AppSettings["connections-file"];
			String connectionsPath = context.Server.MapPath("config/" + idpConnections);
			
			if (File.Exists(connectionsPath)) 
			{
				XmlUtility xmlUtility = new XmlUtility();
				ArrayList connectionsList = xmlUtility.GetPropsFromXmlFile(connectionsPath);
				if(connectionsList == null || connectionsList.Count == 0) 
				{
					String errorMessage = "Error in " + idpConnections;
					context.Response.Redirect(Constants.WEB_PREFIX + Constants.ERROR_PAGE + errorMessage);
				}
				return connectionsList;
			}
			else
			{
				return null;
			}
		}

		public bool IsReusable {
			get { return true; }
		}
	}
}